"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const path = require("path");
const core_1 = require("@stryker-mutator/api/core");
const plugin_1 = require("@stryker-mutator/api/plugin");
const util_1 = require("@stryker-mutator/util");
const BabelConfigReader_1 = require("./BabelConfigReader");
const babel = require("./helpers/babelWrapper");
const helpers_1 = require("./helpers/helpers");
const DEFAULT_EXTENSIONS = babel.DEFAULT_EXTENSIONS;
function babelTranspilerFactory(injector) {
    return injector.provideClass('babelConfigReader', BabelConfigReader_1.BabelConfigReader).injectClass(BabelTranspiler);
}
exports.babelTranspilerFactory = babelTranspilerFactory;
babelTranspilerFactory.inject = plugin_1.tokens(plugin_1.commonTokens.injector);
class BabelTranspiler {
    constructor(options, produceSourceMaps, babelConfigReader) {
        if (produceSourceMaps) {
            throw new Error(`Invalid \`coverageAnalysis\` "${options.coverageAnalysis}" is not supported by the stryker-babel-transpiler. Not able to produce source maps yet. Please set it to "off".`);
        }
        this.babelConfig = babelConfigReader.readConfig(options);
        this.projectRoot = this.determineProjectRoot();
        this.extensions = [...DEFAULT_EXTENSIONS, ...this.babelConfig.extensions];
    }
    async transpile(files) {
        return files.map(file => this.transpileFileIfNeeded(file));
    }
    transpileFileIfNeeded(file) {
        if (this.extensions.some(ext => ext === path.extname(file.name))) {
            try {
                return this.transpileFile(file);
            }
            catch (error) {
                throw new util_1.StrykerError(`Error while transpiling "${file.name}"`, error);
            }
        }
        else {
            return file; // pass through
        }
    }
    transpileFile(file) {
        const relativeOptions = {
            cwd: this.projectRoot,
            filename: file.name,
            filenameRelative: path.relative(this.projectRoot, file.name)
        };
        const options = Object.assign(Object.assign({}, this.babelConfig.options), relativeOptions);
        const result = babel.transformSync(file.textContent, options);
        if (!result) {
            // File is ignored by babel
            return file;
        }
        else if (result.code === undefined || result.code === null) {
            throw new Error(`Could not transpile file "${file.name}". Babel transform function delivered \`undefined\`.`);
        }
        else {
            return new core_1.File(helpers_1.toJSFileName(file.name), result.code);
        }
    }
    determineProjectRoot() {
        const configFile = this.babelConfig.optionsFile;
        if (configFile) {
            return path.dirname(configFile);
        }
        else {
            return process.cwd();
        }
    }
}
exports.BabelTranspiler = BabelTranspiler;
BabelTranspiler.inject = plugin_1.tokens(plugin_1.commonTokens.options, plugin_1.commonTokens.produceSourceMaps, 'babelConfigReader');
//# sourceMappingURL=BabelTranspiler.js.map